#!/usr/bin/python3
# -*- coding: UTF-8 -*-
#
#       主応力の計算
#
#   15/12/17    主応力のベクトル追加
#      12/24    精度向上（有効桁数10桁）、内容を整理
#   16/01/31    reorderSS:初期値を設定
#               Vectorの算出方法を以下のwebを参考に修正
#               http://www.fem.gr.jp/fem/3dim/3dsolidexample/3dsolidexampleb.html
#      04/07    calcVectorSS:特別解を別な方法でvector算出し、主値も再計算
#   18/01/04    python3用に書き換え
#      04/22    vsCodeにより修正
#


import math

SG11 = 0.0; SG12 = 0.0; SG13 = 0.0
SG21 = 0.0; SG22 = 0.0; SG23 = 0.0
SG31 = 0.0; SG32 = 0.0; SG33 = 0.0


#
#  calcSSigma
#    主応力の計算（最大最小は、不確定）
def calcSSigma():
    #不変量
    StInv1 = SG11 + SG22 + SG33
    StInv2 = (-(SG11 * SG22 + SG22 * SG33 + SG33 * SG11) + SG12*SG12 + SG13*SG13 +
                SG23*SG23)
    StInv3 = (SG11 * SG22 * SG33 + 2.0 * SG12 * SG23 * SG13
                - (SG11 * SG23*SG23 + SG22 * SG13*SG13 + SG33 * SG12*SG12))

    #カルノダの方法で3次方程式を解く
    a = 1.0
    b = -StInv1
    c = -StInv2
    d = -StInv3
    p = (3.0 * a * c - b * b) / (9.0 * a*a)
    q = (2.0 * b*b*b - 9.0 * a * b * c + 27.0 * a*a * d) / (27.0 * a*a*a)
    DDD = -(4.0 * p*p*p + q*q)

    SS1 = 0.0
    SS2 = 0.0
    SS3 = 0.0

    if (DDD > 0.0):
        AlpRe = -0.5 * q
        AlpIm = 0.5 * pow(DDD, 0.5)
        R = pow((AlpRe*AlpRe + AlpIm*AlpIm), 0.5)
        if AlpRe == 0.0:
            Theta = 3.1415923238/2.0 * (AlpIm/abs(AlpIm))
        else:
            Theta = math.atan(AlpIm / AlpRe)
        R3 = pow(R, 0.33333333333)
        Theta3 = Theta / 3.0
        Del1 = R3 * math.cos(Theta3)
        Del2 = R3 * math.sin(Theta3)
        if Theta < 0:
            Del1=-Del1
            Del2=-Del2

        kushi1 = 2.0 * Del1
        kushi2 = -Del1 - 1.732050808 * Del2     #sqrt(3)*Del2
        kushi3 = -Del1 + 1.732050808 * Del2

        SS1 = kushi1 - b / (3.0 * a)
        SS2 = kushi2 - b / (3.0 * a)
        SS3 = kushi3 - b / (3.0 * a)
    return [SS1, SS2, SS3]

#
#  calcSVector
#    各方向成分を計算
def calcSVector(SS1, SS2, SS3):

    #  calc_lmn
    #    行列式（2ｘ2）の解
    def calc_lmn(a11, a12, a21, a22):
        #
        # |a11 a12|
        # |a21 a22|
        #
        ans = a11*a22 - a12*a21
        return ans

    #  calcVector
    #    各方向の成分に分解
    def calcVector(l, m, n, sg):
        a = pow((l*l + m*m + n*n), 0.5)
        la = l/a
        ma = m/a
        na = n/a
        SSV = [sg*la, sg*ma, sg*na]
        return SSV

    # def calc_SSlmn(s, lmn):
    #     l = lmn[0]; m = lmn[1]; n = lmn[2]
    #     a = (SG11-s*l) + SG12*m + SG13*n
    #     b = SG21*l + (SG22-s)*m + SG23*n
    #     c = SG31*l + SG32*m + (SG33-s)*n
    #     return a+b+c


    #主軸の方向（方向のみ）
    #       l               m                 n
    # |sg22-sg  sg33 |   |  sg23  sg21|   |sg21 sg22-sg|
    # |  sg32 sg33-sg|   |sg33-sg sg31|   |sg31   sg32 |
    #
    l = calc_lmn(SG22-SS1, SG33,
                 SG32,     SG33-SS1)
    m = calc_lmn(SG23,     SG21,
                 SG33-SS1, SG31)
    n = calc_lmn(SG21,     SG22-SS1,
                 SG31,     SG32)
    SSV1 = calcVector(l, m, n, SS1)

    #       l               m                 n
    # |  sg13   sg12|   |sg11-sg  sg13 |   |sg12 sg11-sg|
    # |sg33-sg  sg32|   | sg31  sg33-sg|   |sg32   sg31 |
    #
    l = calc_lmn(SG13,     SG12,
                 SG33-SS2, SG32)
    m = calc_lmn(SG11-SS2, SG13,
                 SG31,     SG33-SS2)
    n = calc_lmn(SG12,     SG11-SS2,
                 SG32,     SG31)
    SSV2 = calcVector(l, m, n, SS2)

    #       l               m                 n
    # |  sg12   sg13|   |sg13 sg11-sg|   |sg11-sg  sg12 |
    # |sg22-sg  sg23|   |sg23   sg21 |   |sg21   sg22-sg|
    #
    l = calc_lmn(SG12,     SG13,
                 SG22-SS3, SG23)
    m = calc_lmn(SG13,     SG11-SS3,
                 SG23,     SG21)
    n = calc_lmn(SG11-SS3, SG12,
                 SG21,     SG22-SS3)
    SSV3 = calcVector(l, m, n, SS3)
    return [SSV1, SSV2, SSV3]

#
#  calcVectorSS
#    各方向成分を算出
def calcVectorSS(sigmaList, si):
    [xx, yy, zz, xy, yz, xz] = sigmaList

    maxIta = 100
    eps = 1e-14
    flag = 0

    U = []
    for i in range(3):
        # v = []
        # for j in range(3):
        #     v.append(0.0)
        v = [0.0, 0.0, 0.0]
        U.append(v)
    
    for mode in range(3):
        f11 = xx - si[mode]*(1.0-eps)
        f22 = yy - si[mode]*(1.0-eps)
        f33 = zz - si[mode]*(1.0-eps)
        f12 = xy
        f13 = xz
        f23 = yz
        f21 = f12
        f31 = f13
        f32 = f23

        posi = f11*f22*f33 + f21*f32*f13 + f31*f12*f23
        aneg = f13*f22*f31 + f11*f23*f32 + f21*f12*f33
        det = posi - aneg
        if det == 0.0:
            flag = 1
            break 

        g11 =  (f22*f33 - f23*f32)/det
        g12 = -(f21*f33 - f23*f31)/det
        g13 =  (f21*f32 - f22*f31)/det

        g21 = -(f12*f33 - f13*f32)/det
        g22 =  (f11*f33 - f13*f31)/det
        g23 = -(f11*f32 - f12*f31)/det

        g31 =  (f12*f23 - f13*f22)/det
        g32 = -(f11*f23 - f13*f21)/det
        g33 =  (f11*f22 - f12*f21)/det

        g11 = si[mode]*eps*g11
        g12 = si[mode]*eps*g12
        g13 = si[mode]*eps*g13
        g21 = si[mode]*eps*g21
        g22 = si[mode]*eps*g22
        g23 = si[mode]*eps*g23
        g31 = si[mode]*eps*g31
        g32 = si[mode]*eps*g32
        g33 = si[mode]*eps*g33

        U[mode][0] = 0.0
        U[mode][1] = 0.0
        U[mode][2] = 0.0
        U[mode][mode] = 1.0

        vecl = 9999.0
        for i in range(maxIta):
            tempU1 = g11*U[mode][0] + g12*U[mode][1] + g13*U[mode][2]
            tempU2 = g21*U[mode][0] + g22*U[mode][1] + g23*U[mode][2]
            tempU3 = g31*U[mode][0] + g32*U[mode][1] + g33*U[mode][2]

            vecLength = pow((tempU1*tempU1 + tempU2*tempU2 + tempU3*tempU3), 0.5)
            if vecLength == 0.0:
                flag = 1
                break

            U[mode][0] = tempU1/vecLength
            U[mode][1] = tempU2/vecLength
            U[mode][2] = tempU3/vecLength

            delta = abs(vecl - vecLength) / max(vecl, vecLength)

            if delta <= eps:
                break
            vecl =vecLength
        if flag == 1:
            break
    if flag == 0:
        #算出
        for mode in range(3):
            l = abs(si[mode])
            #l = (si[mode])
            for i in range(3):
                U[mode][i] = U[mode][i] * l
    else:
        #算出できない場合（特別解）siも再計算。
        U[0] = [0.0, 0.0, 0.0]
        U[1] = [0.0, 0.0, 0.0]
        U[2] = [0.0, 0.0, 0.0]
        si = [0.0, 0.0, 0.0]
        absValues = [abs(xx), abs(yy), abs(zz)]
        values = sigmaList[:3]
        absMax = max(absValues)
        if absValues[0] == absMax:
            if values[0] > 0:
                U[0][0] = values[0]
                si[0] = values[0]
            else:
                U[2][0] = values[0]
                si[2] = values[0]
        elif absValues[1] == absMax:
            if values[1] > 0:
                U[0][1] = values[1]
                si[0] = values[1]
            else:
                U[2][1] = values[1]
                si[2] = values[1]
        elif absValues[2] == absMax:
            if values[2] > 0:
                U[0][2] = values[2]
                si[0] = values[2]
            else:
                U[2][2] = values[2]
                si[2] = values[2]
                
    return [U[0], U[1], U[2], si]

#
#  reorderSS
#    最大、最小に並べ替え
def reorderSS(SS1, SS2, SS3, SSV1, SSV2, SSV3):
    Sigm1 = SS1; Sigm2 = SS2; Sigm3 = SS3
    sigma1 = SSV1; sigma2 = SSV2; sigma3 = SSV3
    if SS1 > SS2 and SS2 > SS3:
        Sigm1 = SS1; sigma1 = SSV1
        Sigm2 = SS2; sigma2 = SSV2
        Sigm3 = SS3; sigma3 = SSV3
    elif SS1 > SS2 and SS3 > SS2 and SS1 > SS3:
        Sigm1 = SS1; sigma1 = SSV1
        Sigm2 = SS3; sigma2 = SSV3
        Sigm3 = SS2; sigma3 = SSV2
    elif SS2 > SS1 and SS2 > SS3 and SS1 > SS3:
        Sigm1 = SS2; sigma1 = SSV2
        Sigm2 = SS1; sigma2 = SSV1
        Sigm3 = SS3; sigma3 = SSV3
    elif SS2 > SS1 and SS2 > SS3 and SS3 > SS1:
        Sigm1 = SS2; sigma1 = SSV2
        Sigm2 = SS3; sigma2 = SSV3
        Sigm3 = SS1; sigma3 = SSV1
    elif SS2 > SS1 and SS3 > SS2:
        Sigm1 = SS3; sigma1 = SSV3
        Sigm2 = SS2; sigma2 = SSV2
        Sigm3 = SS1; sigma3 = SSV1
    elif SS3 > SS1 and SS1 > SS2:
        Sigm1 = SS3; sigma1 = SSV3
        Sigm2 = SS1; sigma2 = SSV1
        Sigm3 = SS2; sigma3 = SSV2
    return [Sigm1, Sigm2, Sigm3, sigma1, sigma2, sigma3]


#
#  pricpalStress
#  -------------
#    主応力の計算
#    入力:[xx, yy, zz, xy, yz, zx]
#    戻り:[sigma1, sigma2, sigma3, vectorSigma1, vectorSigma2, vectorSigma3]
def principalStress(sigmaList):

    global SG11, SG12, SG13
    global SG21, SG22, SG23
    global SG31, SG32, SG33

    #xx,   yy,   zz,   xy,   yz,   zx
    [SG11, SG22, SG33, SG12, SG23, SG13] = sigmaList
    SG21 = SG12
    SG31 = SG13
    SG32 = SG23
    #3ヶの実根を求める
    [SS1, SS2, SS3] = calcSSigma()
    #各方向の成分を求める
    #[SSV1, SSV2, SSV3] = calcSVector(SS1, SS2, SS3)
    [SSV1, SSV2, SSV3, [SS1, SS2, SS3]] = calcVectorSS(sigmaList, [SS1, SS2, SS3])
    #最大、最小を求める（並べ替え）
    [SS1, SS2, SS3, SSV1, SSV2, SSV3] = reorderSS(SS1, SS2, SS3, SSV1, SSV2, SSV3) 
    return [SS1, SS2, SS3, SSV1, SSV2, SSV3]


if __name__ == "__main__":
    #xx=4.61E+02; yy=1.50E+02; zz=2.07E+02
    #xy=-3.13E+01; yz=-1.15E+01; zx=-6.62E+01

    #sigmaList = [xx, yy, zz, xy, yz, zx]
    #sigmaList = [10000000.000471603, 0.00047178906250000004, 1.353400902866366e-24, 0.0004718398437500004, -4.204932183080673e-09, -4.204932183074549e-09]
    sigmaList = [10000000.000471603, 0.00047178906250000004, 1e-11, 0.0004718398437500004, -4.204932183080673e-09, -4.204932183074549e-09]

    [sig1, sig2, sig3, v1, v2, v3] = principalStress(sigmaList)
    mises2 = (pow((sig1-sig2),2) + pow((sig2-sig3),2) + pow((sig3-sig1),2))/2.0
    mises = pow(mises2, 0.5)
    print (sig1, sig2, sig3, [v1, v2, v3])

