#!/usr/bin/python3  
# -*- coding: utf-8 -*-    
#
#       runCommandWithDialog.py
#
#   使い方
#   runCommandWithDialog.py <job> <command>
#       job     :job名
#       command :実行するコマンド
#
#   job名からtitle, messageを取得し、commandを実行する
#       
#
#   15/06/19    新規作成
#      09/02    "\n"→os.linesepに修正
#      10/18    messageをjobに応じて作成する様に修正
#      12/18    windows版のpangoWarning対策で、初期化を修正
#      12/25    行末コードを修正
#   19/01/07    python3、Gtk+3用に書き換え
#      01/21    dialogを修正。
#      01/26    runCommand:windows用に修正
#               汎用dialog:type→message_typeに修正
#               subprocess起動と中断方法をwindowsとlinuxで分ける
#
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
#from gi.repository import GObject
from gi.repository import GLib

import sys   
import os
import threading
import subprocess
import signal
import gettext
gettext.install("app") # replace with the appropriate catalog name
_ = gettext.gettext

title = ""
message = ""
command = ""
ls = "\n"
outLines = []
closeFlag = 0
stopFlag = 0

#  job名とその内容
jobDict = {
    #job名               title               message
    "convData":     [_(u"データ変換"),   _(u"データ変換が完了しました")],
    "mergeResult":  [_(u"結果の再構築"), _(u"分散結果をマージしました。")],
    "separateMesh": [_(u"メッシュ分割"), _(u"メッシュ分割\nメッシュを分割しました。")],
    "":             ["", ""]
    }


#----------------
#  windowApp
#----------------
#    GUIの起動と終了
class windowApp:  

    #  GUIの初期化
    def __init__(self):
        global command

        gladefile = "runCommandWithDialog2.glade"  
        self.builder = Gtk.Builder()
        self.builder.add_from_file(os.path.dirname(os.path.abspath(__file__)) +
                os.sep + gladefile)  
        self.setGtkObject()
        self.subInitialize()

        self.builder.connect_signals(self)
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.mainWindow.set_title(title)
        #GObject.timeout_add(200, self.timerEvent)
        GLib.timeout_add(200, self.timerEvent)

    #  GUIの表示
    def main(self):
        self.mainWindow.show_all()
        self.runThread()
        Gtk.main()

    #  GUIの終了
    def close(self, *args):
        Gtk.main_quit(*args)

    #  comm停止
    def onCancel(self, event):
        global closeFlag, stopFlag
        mess = title + u"の中断\n"
        mess += u"本当に中断しますか?"
        stat = self.okCancelDialog(title, mess)
        if stat == "OK":
            if os.name == "nt":
                #windowsの場合
                #errorコードを送出
                try:
                    dummy_outs, dummy_errs = self.proc.communicate(timeout=0.1)
                except:
                    self.proc.kill()
                    dummy_outs, dummy_errs = self.proc.communicate()
            else:
                #linuxの場合
                #全てのpidを停止
                os.killpg(os.getpgid(self.proc.pid), signal.SIGTERM)
            #中断flagを設定
            stopFlag = 1

    #  object名を取得
    def setGtkObject(self):
        self.label_mess = self.builder.get_object("label1")
        self.label_out = self.builder.get_object("label2")

    #  初期化
    def subInitialize(self):
        self.label_mess.set_text(title + u"中...")

    #  timerEvent
    def timerEvent(self):
        global outLines
        lines = outLines[-4:]
        cont = "".join(lines)
        self.label_out.set_text(cont)
        if closeFlag == 0 and stopFlag == 0:
            #終了していない場合は、timerEvent再設定
            #GObject.timeout_add(50, self.timerEvent)
            GLib.timeout_add(50, self.timerEvent)
        elif closeFlag == 1 and stopFlag == 0:
            #終了している場合、終了メッセージを出力
            self.okDialog(title, message)
            Gtk.main_quit()
        else:
            #中断の場合
            mess = title + _(u" を中断しました")
            print (mess)
            self.okDialog(title, mess)
            Gtk.main_quit()

    #
    #  runThread
    #    thread実行
    def runThread(self):
        thread = threading.Thread(target=self.runCommand)
        thread.start()

    #  runCommand
    #    command実行
    #    正常終了した場合は、closeFlag=1 に設定する
    def runCommand(self):
        global outLines, command, closeFlag
        print ()
        print (command)
        outLines = [title+ls, ls, command + ls]
        command = command + " 2>&1"
        #subProcessを起動
        if os.name == "nt":
            #windows
            self.proc = subprocess.Popen(
                        command,                    #コマンド
                        shell=True,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.PIPE)
            try:
                for line in iter(self.proc.stdout.readline, b""):
                    line = line.decode("shift-jis")
                    outLines.append(line)
                    print (line, end=" ")
                closeFlag = 1
            except:
                pass
        else:
            #linux
            #「preexec_fn=os.setsid」でpidのgroupを作成（後で全て停止できるように）
            self.proc = subprocess.Popen(
                        command,                    #コマンド
                        shell=True,
                        stdout=subprocess.PIPE,
                        stderr=subprocess.PIPE,
                        preexec_fn=os.setsid)
            #subProcessの出力をrealTimeで取得する
            for line in iter(self.proc.stdout.readline, b""):
                line = line.decode("utf-8")
                outLines.append(line)
                print (line, end=" ")
            """
            self.proc.stdout.flush()
            for line in self.proc.stdout:
                self.proc.stdout.flush()
                line = line.decode("utf-8")
                outLines.append(line)
                print (line, end=" ")
            """
            closeFlag = 1

    def okDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                parent=self.mainWindow,
                message_type=Gtk.MessageType.INFO,
                buttons=Gtk.ButtonsType.OK)
        dlg.set_title(title)
        #messageを分ける
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        dlg.run() 
        dlg.destroy()

    def errDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                parent=self.mainWindow,
                message_type=Gtk.MessageType.ERROR,
                buttons=Gtk.ButtonsType.OK)
        dlg.set_title(title)
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        dlg.run() 
        dlg.destroy()

    def okCancelDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                parent=self.mainWindow,
                message_type=Gtk.MessageType.INFO,
                buttons=Gtk.ButtonsType.OK_CANCEL)
        dlg.set_title(title)
        #messageを分ける
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        ans = dlg.run() 
        dlg.destroy()
        if ans == Gtk.ResponseType.OK:
            stat = "OK"
        else:
            stat = "CANCEL"
        return stat


#
#  getTitleMessage
#    jobからtitleとmessageを決定
def getTitleMessage(job):
    global title, message
    if job in jobDict.keys():
        title, message = jobDict[job]
    else:
        title, message  = ""


if __name__ == "__main__":  
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    job = sys.argv[1]
    command = sys.argv[2]

    getTitleMessage(job)

    app = windowApp()
    app.main()  

